﻿//------------------------------------------------------------------------------
//
// TPGLibrary.jsx
//
// A library of common routines used by Javascript components of TPG Lightroom
// plugins.
//
// More information available at: http://thephotogeek.com/lightroom/
//
//------------------------------------------------------------------------------
//
// Copyright 2010-2013 Matt Dawson.  All rights reserved.
//
//------------------------------------------------------------------------------


//=========================================
// Constants
//


//=========================================
// Globals
//


//=========================================
// Utility functions
//

// cTID: Abbreviation for charIDToTypeID
function cTID(str) {
    return charIDToTypeID(str);
}

// sTID: Abbreviation for stringIDToTypeID
function sTID(str) {
    return stringIDToTypeID(str);
}


//=========================================
// TPGError
//
// Assists with processing errors.
//
// Return: TPGError object
// Example:
//
//   var a = new TPGError();
//   a.isError( 8007 );
//
function TPGError() {
	// member properties

	// known errors with more friendly messages for each
    this.knownError = {
// e.g.       8000: { userMessage: "dialog messgae", logMessage: "log file message"},
// 8000/8001 - Camera Raw open failure
// 8800 - feature unavailable e.g. the PSE 11 Editor is not in Expert mode when copying a layer
    };

    // non-errors that should be igmored
    this.nonError = {
        8007: true // User cancelled error
    };

	// member methods


	// Get the log file message associated with this error
	this.getLogMessage = function(err) {
	    var result = "";

	    if (this.isKnown(err.number)) {
	        result = this.knownError[err.number].logMessage;
	    } else {
	        result = err.number + ": " + err;
	    };

        return result;
	};

	// Get the user message associated with this error
	this.getUserMessage = function(err) {
	    var result = "";

	    if (this.isKnown(err.number)) {
	        result = this.knownError[err.number].userMessage;
	    } else {
	        result = err.number + ": " + err;
	    };

        return result;
	};

	// Determines if this error is really a non-error
	this.isError = function(err) {
        return !(err.number in this.nonError);
	};

	// Determines if this error is known
	this.isKnown = function(err) {
        return (err.number in this.knownError);
	};
};


//=========================================
// TPGImage
//
// Image processing.
//
// Return: TPGImage object
//
function TPGImage() {
	// member properties


	// member methods


    // openRawWithSettings(fileObj, bitDepth)
    //
    // Adapted from:
    //   open raw files and handle jpeg as raw
    //   http://ps-scripts.com/bb/viewtopic.php?f=9&t=2468&p=10588
	this.openRawWithSettings = function(fileObj, bitDepth) {
        var fileDesc = new ActionDescriptor();
        var optionsDesc = new ActionDescriptor();

        fileDesc.putPath(sTID('null'), fileObj);

        // Only set the bit depth if a specific bit depth was configured
        if (bitDepth == 8) {
            optionsDesc.putEnumerated(cTID('BtDp'), cTID('BtDp'), cTID('BD8 '));
        }
        else if (bitDepth == 16) {
            optionsDesc.putEnumerated(cTID('BtDp'), cTID('BtDp'), cTID('BD16'));
        }

        optionsDesc.putEnumerated(cTID('Sett'), cTID('Sett'), cTID('Img '));
        fileDesc.putObject(cTID('As  '), sTID("Adobe Camera Raw"), optionsDesc);
        return executeAction(cTID('Opn '), fileDesc, DialogModes.NO);
    };
};


//=========================================
// TPGLogger
//
// Log information to a text file.
//
// Params: String to full path of file to create or append, if no file is given
//        then output file TPGLogger.log is created on the users desktop.
// Return: TPGLogger object
// Example:
//
//   var a = new TPGLogger();
//   a.log( Date() );
//   a.display();
//   a.remove();
//
function TPGLogger( inFile ) {
	// member properties

	// the file we are currently logging to
	if ( undefined == inFile ) {
		this.file = new File( Folder.myDocuments + "/TPGLogger.log" );
	} else {
		this.file = new File( inFile );
	};

	// member methods


	// write out a message to the log file.
	// Non-thread safe.
	this.logQuick = function( inMessage ) {
	    var result = false;

		if ( this.file.exists ) {
			this.file.open( 'e' );
			this.file.seek( 0, 2 ); // end of file
		} else {
			this.file.open( 'w' );
		};

		// Repeated exists check as an external process might interfere with the
		// file prior to writing
		if ( this.file.exists ) {
		    result = this.file.write( inMessage );
		};
		this.file.close();

		return result;
	};

	// write out a message to the log file.
	// Retries once if log message failed to write the first time.
	this.log = function( inMessage ) {
		// Repeated log attempt as an external process might interfere with the
		// file while attempting to write. Main culprit would be the pipe files
		// so a double attempt to write should be sufficient.
		if ( !this.logQuick(inMessage) ) {
			this.logQuick(inMessage);
		};
	};

	// write out a message to the log file, prefixed by time stamp
	// TODO: Implement TPGLogger.logTimed
	this.logTimed = function( inMessage ) {
		this.log(inMessage);
	};

	// show the contents with the execute method
	this.display = function() {
		this.file.execute();
	};

	// remove the file
	this.remove = function() {
		this.file.remove();
	};
};


//=========================================
// TPGPipe
//
// Log messages to a pipe file. Used as a basic IPC mechanism.
//
// Params: String to full path of pipe file to create or append, if no file is given
//        then output file TPGPipe.log is created on the users desktop.
// Return: TPGPipe object
// Example:
//
//   var a = new TPGPipe();
//   a.write( "Camera Raw", 7.3 );
//
function TPGPipe( inFile ) {
	// member properties

	// the pipe we are currently writing to
	if ( undefined == inFile ) {
		this.pipe = new TPGLogger( Folder.myDocuments + "/TPGPipe.log" );
	} else {
		this.pipe = new TPGLogger( inFile );
	};

	// member methods


	// Close the pipe
	this.close = function() {
		this.write("CLOSE", "true");
	};

	// Initialise the pipe's content
	this.initialise = function() {
        var systemInfo = new TPGSystemInfo();

		this.pipe.remove();
		this.write("PSEVERSION", systemInfo.getApplicationVersion());
		this.write("CAMERARAWVERSION", systemInfo.getCameraRawVersion());
	};

	// Write a caught error to the pipe
	this.writeError = function(err) {
		this.writeErrorDetail(err.number, err);
	};

	// Write the detail of an error to the pipe
    this.writeErrorDetail = function(errnum, errmsg) {
		this.write("ERRORNUMBER", errnum);
		this.write("ERRORMESSAGE", errmsg);
	};

	// write out a message to the pipe
	this.write = function( name, val ) {
        var inMessage = name + "=[[" + val + "]]\n";

		this.pipe.log( inMessage );
	};
};


//=========================================
// TPGSystemInfo
//
// Accesses system information.
//
// Params: None.
// Return: TPGSystemInfo object
// Example:
//
//   var a = new TPGSystemInfo();
//
function TPGSystemInfo() {
	// member properties


	// member methods


	// TBD
	this.getApplicationLocale = function() {
		return app.locale;
	};

	// TBD
	this.getApplicationName = function() {
		return app.name;
	};

	// TBD
	this.getApplicationVersion = function() {
		return app.version;
	};

	// TBD
	this.getBridgeTalkName = function() {
		return BridgeTalk.appName;
	};

	// Used to retrieve the Camera Raw plugin version.
	this.getCameraRawVersion = function() {
		var version = 0

		// if Camera Raw information can be found via systemInformation
		if ((app.systemInformation.search(/Camera.Raw/) > 0)) {
			version = app.systemInformation.match(/Camera.Raw.([0-9]*\.[0-9]*)/)[1];
		}

		return version;
	};

	// Used to retrieve the Camera Raw version information written to the
	// document's XMP metadata.
	this.getCameraRawVersionFromDoc = function(currentDocument) {
		var version = 0
		
		// if this was a document opened via camera raw
		if (undefined != currentDocument
			&& undefined != currentDocument.xmpMetadata) {
			version = currentDocument.xmpMetadata.rawData.match(/<crs:Version>(.+)<\/crs:Version>/)[1];
		}
	 
		return version;
	};

	// Used to retrieve Photoshop Elements' Camera Raw plugin version by
	// opening an image (e.g. JPEG) with no XMP metadata in Camera Raw.
	// This forces generation of XMP metadata including its default crs:version.
	this.getCameraRawVersionFromFile = function(fileName) {
		var currentDocument;
		var errorMessage = "";
		var version = 0;
		
		try {
			currentDocument = open(new File(fileName), new CameraRAWOpenOptions);
			version = this.getCameraRawVersionFromDoc(currentDocument);
			currentDocument.close(SaveOptions.DONOTSAVECHANGES);
		}
		catch(e) {
			errorMessage = e + '@' + e.line;
		};

		if (version > 0) {
			return version;
		} else {
			return errorMessage;
		}
	};

	// TBD
	this.getOs = function() {
		return $.os;
	};

	// TBD
	this.getOsName = function() {
		var os = "";

		if($.os.indexOf("ista") != -1) { os = "Vista" };
		if($.os.indexOf("acint") != -1) { os = "Mac OSX" };
		if($.os.indexOf("XP") != -1) { os = "XP" };
		if($.os.indexOf("indows 7") != -1) { os = "Windows 7" };
		if($.os.indexOf("indows 6.1") != -1) { os = "Windows 7" };
		if($.os.indexOf("indows 8") != -1) { os = "Windows 8" };
		if($.os.indexOf("indows 6.2") != -1) { os = "Windows 8" };

		return os;
	};

    this.getLogIdentity = function() {
        var id = "\n--------------------------------------------------------------------\n"
            + "Application: '" + this.getBridgeTalkName()
            + "': '" + this.getApplicationName()
            + "' (" + this.getApplicationVersion() + ")\n"
            + "Camera Raw: " + this.getCameraRawVersion() + "\n"
            + "OS: " + this.getOsName() + " (" + this.getOs() + ")\n"
            + "Locale: " + this.getApplicationLocale() + "\n"
            + "---------------------------------------------------------------------\n\n";

        return id;
    };
};

//EOF
